from typing import Type, Any, Dict, List
from inspect import Signature, signature
from ..enums import Scope
from ..interfaces import IScopeManager
from ..dtos import Registration
from ..helpers import TypeHelpers

class TransientScopeManager(IScopeManager):
    __base_types: List[Type]
    __ignore_parameters: List[str]

    def __init__(self, base_types: List[Type]) -> None:
        self.__base_types = base_types
        self.__ignore_parameters = [
            'self',
            'args',
            'kwargs'
        ]
    def can_resolve(self, scope: Scope) -> bool:
        return scope == Scope.TRANSIENT

    def is_same_registration_scope(self, interface: Type, container: Dict[Type, Registration]) -> bool:
        registration: Registration = container[interface]
        return registration.scope == Scope.TRANSIENT

    def resolve(self, interface: Type, container: Dict[Type, Registration]) -> Any:
        error_message: str = ''
        if interface in container.keys():
            implementation: Any = container[interface].implementation
            if(type(implementation) in self.__base_types):
                kwargs: Any = {}
                sig: Signature = signature(implementation)
                for p in sig.parameters:
                    if(p not in self.__ignore_parameters):
                        child_interface: Type = sig.parameters[p].annotation
                        if not self.is_same_registration_scope(interface=child_interface, container=container):
                                error_message = f'Error Singleton type: {TypeHelpers.to_string(interface)} registered with Transient dependency: {TypeHelpers.to_string(child_interface)}'
                                raise Exception(error_message)
                        instance = self.resolve(interface= child_interface, container= container)
                        kwargs[p] = instance
                return implementation(**kwargs)
            elif(type(implementation) == type(lambda: '')):
                return implementation()
            else:
                return implementation
        else:
            error_message = f'Cannot resolve type: {TypeHelpers.to_string(interface)}'
            raise Exception(error_message);
